/*-------------------------------------------------------------------
Name......... cctalkbill.h
Author....... Jon Carroll
Date......... 24/04/07
Description.. Header file for generic bill driver.
-------------------------------------------------------------------*/
#ifndef PERIPHERAL_ARDAC
#define PERIPHERAL_ARDAC
/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

#define TOTAL_BILLS		64

typedef	struct cctalkbillcfg
{
	UBYTE SlavePort;
	UBYTE SlaveAddress;
	const SECURITY_KEY *TheKey;

} CCTALKBILLCFG;

typedef struct
{

	UWORD Counts;
	WORD Checksum;

} BILL_VALUE;


typedef struct
{
								/* the event number */
	UBYTE EventNumber;
								/* the bill values */
	BILL_VALUE BillCounts[TOTAL_BILLS];

								/* The Error values */
	ULONG Error;

} BILL_DATA;




typedef	struct
{
								/* Enable the bill processing */
	void (*enable_bill_processing)(const DEVICE* dev);

								/* disable the bill processing */
	void (*disable_bill_processing)(const DEVICE* dev);

								/* get bill error */
	UBYTE (*get_bill_errors)(const DEVICE* dev);

								/* enable all bills */
	void (*enable_all_bills)(const DEVICE* dev);

								/* Enable Bill */
	void (*enable_bill)(const DEVICE* dev, UBYTE Bill);

								/* Disable all bills */
	void (*disable_all_bills)(const DEVICE* dev);

								/* Disable Bill */
	void (*disable_bill)(const DEVICE* dev, UBYTE Bill);

								/* is bill disabled */
	BYTE (*is_bill_enabled)(const DEVICE *dev, UBYTE Bill);

								/* Error on stacker full */
	void (*set_error_on_stacker_full)(const DEVICE* dev, UBYTE Value);

								/* get the status of error */
								/* on stacker full */
	BYTE (*get_error_on_stacker_full)(const DEVICE* dev);

								/* is stacker removed */
	BYTE (*is_stacker_removed)(const DEVICE* dev);

								/* get bill counts */
	UWORD (*get_bill_count)(const DEVICE* dev, UBYTE Bill);

								/* subtract from bill counts */
	void (*subtract_from_bill_counts)(const DEVICE* dev, UBYTE Bill);

								/* Add to bill counts */
	void (*add_to_bill_count)(const DEVICE *dev, UBYTE Bill);

								/* set the polling rate */
	void (*set_bill_polling_rate)(const DEVICE *dev, UWORD Rate);

} BILLSERVICES;

/* Error event codes							*/
enum
{
	BILL_ERROR_NO_ERROR,
	BILL_ERROR_LOST_EVENT,
	BILL_ERROR_LOST_DEVICE,
	BILL_ERROR_STACKER_FULL,
	BILL_ERROR_STACKER_FAULT,

	TOTAL_ERRORS_BILL_ERROR
};


#include "cctalkbill.p"

#endif