/*-------------------------------------------------------------------
Name......... cctalkhal.h
Author....... Jon Carroll
Date......... 23/04/07
Description.. Header file for cctalk driver.
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

#ifndef CCTALKHAL_H
#define CCTALKHAL_H



	/* Decide which modules to compile in:									*/
	#define CCTALK_CORE								TRUE
	#define CCTALK_COREPLUS							TRUE
	#define CCTALK_MULTIDROP						TRUE
	#define CCTALK_ACCEPTOR							TRUE
	#define CCTALK_VALIDATOR						TRUE
	#define CCTALK_PAYOUT							TRUE

	/* Not recommended, however include legacy commands						*/
	#define CCTALK_OBSOLETE							TRUE

	/* Advanced Compile options to remove  cctalk headers from the library	*/

	/* Required to strip out functions pairs that are not needed			*/
	#define THIN_AND_LEAN							FALSE

	/* A define for every pair of transmit/receive functions				*/

	/* cctalk Core functions												*/
	#define ENABLE_SIMPLEPOLL						TRUE
	#define ENABLE_MANUFACTURERID					TRUE
	#define ENABLE_EQUIPMENTCATEGORYID				TRUE
	#define ENABLE_PRODUCTCODE						TRUE
	#define ENABLE_BUILDCODE						TRUE

	/* cctalk Core Plus functions											*/
	#define ENABLE_SERIALNUMBER						TRUE
	#define ENABLE_COMMSREVISION					TRUE
	#define ENABLE_CLEARCOMMSSTATUSVARIABLES		TRUE
	#define ENABLE_COMMSSTATUSVARIABLES				TRUE
	#define ENABLE_RESET							TRUE

	/* cctalk Multi-drop functions											*/
	#define ENABLE_ADDRESSPOLL						TRUE
	#define ENABLE_ADDRESSCLASHCHANGE				TRUE
	#define ENABLE_ADDRESSCHANGE					TRUE
	#define ENABLE_ADDRESSRANDOMCHANGE				TRUE

	/* cctalk Obsolete functions											*/
	#define ENABLE_READLASTCREDITORERRORCODE		TRUE
	#define ENABLE_ISSUEGUARDCODE					TRUE
	#define ENABLE_DISPENSECOINS					TRUE
	#define ENABLE_DISPENSECHANGE					TRUE
	#define ENABLE_ONESHOTCREDIT					TRUE
	#define ENABLE_EMPTYPAYOUT						TRUE
	#define ENABLE_AUDITINFORMATIONBLOCK			TRUE
	#define ENABLE_UPLOADCOINDATA					TRUE
	#define ENABLE_PAYOUTSTATUS						TRUE

	/* cctalk Acceptor functions											*/
	#define ENABLE_POLLINGPRIORITY					TRUE
	#define ENABLE_REQUESTSTATUS					TRUE
	#define ENABLE_REQUESTVARIABLESET				TRUE
	#define ENABLE_REQUESTDATABASEVERSION			TRUE
	#define ENABLE_TESTSOLENOIDS					TRUE
	#define ENABLE_TESTOUTPUTLINES					TRUE
	#define ENABLE_READINPUTLINES					TRUE
	#define ENABLE_READOPTOSTATES					TRUE
	#define ENABLE_LATCHOUTPUTLINES					TRUE
	#define ENABLE_PERFORMSELFCHECK					TRUE
	#define ENABLE_MODIFYINHIBITSTATUS				TRUE
	#define ENABLE_INHIBITSTATUS					TRUE
	#define ENABLE_READBUFFEREDCREDITORERRORCODES	TRUE
	#define ENABLE_MODIFYMASTERINHIBITSTATUS		TRUE
	#define ENABLE_MASTERINHIBITSTATUS				TRUE
	#define ENABLE_INSERTIONCOUNTER					TRUE
	#define ENABLE_ACCEPTCOUNTER					TRUE
	#define ENABLE_MODIFYSORTEROVERRIDESTATUS		TRUE
	#define ENABLE_SORTEROVERRIDESTATUS				TRUE
	#define ENABLE_ENTERNEWPINNUMBER				TRUE
	#define ENABLE_EQUIPMENTCATEGORYID				TRUE
	#define ENABLE_SOFTWAREREVISION					TRUE
	#define ENABLE_DATASTORAGEAVAILABILITY			TRUE
	#define ENABLE_READDATABLOCK					TRUE
	#define ENABLE_WRITEDATABLOCK					TRUE
	#define ENABLE_OPTIONFLAGS						TRUE
	#define ENABLE_COINPOSITION						TRUE
	#define ENABLE_MODIFYSORTERPATHS				TRUE
	#define ENABLE_SORTERPATHS						TRUE
	#define ENABLE_TEACHMODECONTROL					TRUE
	#define ENABLE_TEACHSTATUS						TRUE
	#define ENABLE_CONFIGURATIONTOEEPROM			TRUE
	#define ENABLE_COUNTERSTOEEPROM					TRUE
	#define ENABLE_CREATIONDATE						TRUE
	#define ENABLE_LASTMODIFICATIONDATE				TRUE
	#define ENABLE_REJECTCOUNTER					TRUE
	#define ENABLE_FRAUDCOUNTER						TRUE
	#define ENABLE_MODIFYDEFAUTLSORTERPATHS			TRUE
	#define ENABLE_DEFAULTSORTERPATHS				TRUE
	#define ENABLE_MODIFYCOINID						TRUE
	#define ENABLE_COINID							TRUE
	#define ENABLE_UPLOADWINDOWDATA					TRUE
	#define ENABLE_DOWNLOADCALIBRATIONINFO			TRUE
	#define ENABLE_MODIFYSECURITYSETTING			TRUE
	#define ENABLE_SECURITYSETTING					TRUE
	#define ENABLE_MODIFYBANKSELECT					TRUE
	#define ENABLE_BANKSELECT						TRUE
	#define ENABLE_ALARMCOUNTER						TRUE
	#define ENABLE_THERMISTORREADING				TRUE
	#define ENABLE_BASEYEAR							TRUE
	#define ENABLE_MODIFYINHIBITOVERRIDEREGISTERS	TRUE
	#define ENABLE_SETACCEPTLIMIT					TRUE

	/* cctalk Validator functions											*/
	#define ENABLE_OPERATEMOTORS					TRUE
	#define ENABLE_READBUFFEREDBILLEVENTS			TRUE
	#define ENABLE_MODIFYBILLID						TRUE
	#define ENABLE_BILLID							TRUE
	#define ENABLE_COUNTRYSCALINGFACTOR				TRUE
	#define ENABLE_BILLPOSITION						TRUE
	#define ENABLE_ROUTEBILL						TRUE
	#define ENABLE_MODIFYBILLOPERATINGMODE			TRUE
	#define ENABLE_BILLOPERATINGMODE				TRUE
	#define ENABLE_TESTLAMPS						TRUE
	#define ENABLE_INDIVIDUALACCEPTCOUNTER			TRUE
	#define ENABLE_INDIVIDUALERRORCOUNT				TRUE
	#define ENABLE_READOPTOVOLTAGES					TRUE
	#define ENABLE_PERFORMSTACKERCYCLE				TRUE
	#define ENABLE_OPERATEBIDIRECTIONALMOTORS		TRUE
	#define ENABLE_CURRENCYREVISION					TRUE
	#define ENABLE_UPLOADBILLTABLES					TRUE
	#define ENABLE_BEGINBILLTABLES					TRUE
	#define ENABLE_FINISHBILLTABLEUPGRADE			TRUE
	#define ENABLE_FIRMWAREUPGRADE					TRUE
	#define ENABLE_UPLOADFIRMWARE					TRUE
	#define ENABLE_BEGINFIRMWAREUPGRADE				TRUE
	#define ENABLE_FINISHFIRMWAREUPGRADE			TRUE
	#define ENABLE_SWITCHENCRYPTIONCODE				TRUE
	#define ENABLE_STOREENCRYPTIONCODE				TRUE

	/* cctalk Payout functions												*/
	#define ENABLE_MODIFYPAYOUTABSOLUTECOUNT		TRUE
	#define ENABLE_PAYOUTABSOLUTECOUNT				TRUE
	#define ENABLE_MODIFYPAYOUTCAPACITY				TRUE
	#define ENABLE_PAYOUTCAPACITY					TRUE
	#define ENABLE_MODIFYPAYOUTFLOAT				TRUE
	#define ENABLE_PAYOUTFLOAT						TRUE
	#define ENABLE_EMERGENCYSTOP					TRUE
	#define ENABLE_HOPPERCOIN						TRUE
	#define ENABLE_HOPPERDISPENSECOINS				TRUE
	#define ENABLE_DISPENSEHOPPERCOINS				TRUE
	#define ENABLE_HOPPERSTATUS						TRUE
	#define ENABLE_MODIFYVARIABLESET				TRUE
	#define ENABLE_ENABLEHOPPER						TRUE
	#define ENABLE_TESTHOPPER						TRUE
	#define ENABLE_PUMPRNG							TRUE
	#define ENABLE_CIPHERKEY						TRUE


	#define CCTALK_DATA_PACKET						252

	#define CCTALK_MASTER_ADDRESS					1

	#define TOTAL_RETRIES							3

	/* CRC Methods and encryption available								*/
	#define	CCTALK_CRC8								0x0
	#define	CCTALK_CRC16							0x1
	#define	CCTALK_BNV								0x2

								/* the maximum inter-characater timeout */
	#define INTER_CHAR_TIMEOUT						100

								/* the inter message timeout */
	#define INTER_MESSAGE_TIMEOUT					2000

								/* defines for the cctalk NAK and BUSY
								   messages */
	#define BUSY_MESSAGE							6
	#define NAK_MESSAGE								5

								/* the size of the buffers for  */
								/* the ccTalk messages*/
	#define CCTALK_BUFFER_SIZE						256

								/* the minimum cctalk message size */
	#define MINIMUM_CCTALK_MESSAGE					5

	#define CCTALK_HEADER_SIZE						3

	#define SIZE_OF_UBYTE							255

	enum
	{
		CCTALK_PORT_1,
		CCTALK_PORT_2,
		TOTAL_CCTALK_PORTS
	};

	typedef struct timer2
	{

		ULONG expires;

	} TIMER2;


	/* The device manager configuration structure */
	typedef	struct cctalkcfg
	{
								/* the cctalk master address */
		UBYTE masteraddress;
								/* first cctalk serial device */
		const DEVICE* port1;
		BYTE channel1;
								/* second cctalk serial device */
		const DEVICE* port2;
		BYTE channel2;

	} CCTALKCFG;


								/* The security key type definition */
	typedef struct
	{
		UBYTE crcmode;
		UBYTE key[6];
	} SECURITY_KEY;


								/* The definition for modifing current status
								   of the ccTalk Port */
	typedef struct
	{
								/* the physical device */
		const DEVICE* device;
								/* the physical channel */
		BYTE channel;
								/* the the current status of the */
		UBYTE status;
								/* is the ccTalk Bus locked */
		UBYTE locked;
								/* is the ccTalk port opened */
		BYTE opened;
								/* the current state machine */
		UBYTE state;
								/* the current number of byte to transmit or
								    receive */
		UBYTE numbytes;
								/* pointer to the current byte to send or receive */
		UBYTE *ptr;
								/* Number of retries */
		UBYTE retries;
								/* Security key */
		SECURITY_KEY securitykey;
								/* general purpose timer */
		TIMER2 timer;

								/* the buffer for the Transmit buffer */
		UBYTE txbuffer[CCTALK_BUFFER_SIZE];
								/* buffer for the recieve buffer */
		UBYTE rxbuffer[CCTALK_BUFFER_SIZE];

	} CCTALK_PORT_VARS;


	typedef struct
	{
								/* pointer to the configuration */
		const CCTALKCFG* 	cfg;
								/* the control structures for the cctalk channels */
		CCTALK_PORT_VARS port[TOTAL_CCTALK_PORTS];

	} CCTALK_VARS;

								/* A cctalk packet */
	typedef struct
	{
		UBYTE dest;
		UBYTE numbytes;
								/* also the LSB of the */
								/* CRC 16 checksum */
		UBYTE source;
		UBYTE header;
		UBYTE data[CCTALK_DATA_PACKET];
		SECURITY_KEY crcdata;
	} CCTALK_PACKET;


	typedef struct
	{
		UBYTE datalength;
		UBYTE data[255];
	} CCTALK_VARIABLE;

	typedef struct
	{
		UBYTE curmask1;
		UBYTE curmask2;
		UBYTE curbitmask;
		UBYTE nexmask1;
		UBYTE nexmask2;
		UBYTE nexbitmask;
	} CCTALK_INHIBITS;

	typedef struct
	{
		UBYTE memorytype;
		UBYTE readblocks;
		UBYTE readbytesperblock;
		UBYTE writeblocks;
		UBYTE writebytesperblock;
	} CCTALK_STORAGE;

	enum
	{
		CCTALK_STATUS_IDLE,			/* Can transmit on the bus */
		CCTALK_STATUS_BUSY,			/* The bus is currently busy */
		CCTALK_STATUS_RECIEVE,		/* Data is ready to be recieved */
		CCTALK_STATUS_ERROR			/* there is an error */
	};


	typedef union
	{
		UBYTE byte[2];
		UWORD word;
	} WORD_AND_BYTES;

	typedef struct
	{
		UBYTE eventcounter;
		UBYTE result[5];
		UBYTE resulttype[5];
	} CCTALK_EVENTS;

#include "cctalkhal.p"

#endif	/* HAL_H			*/