/*-------------------------------------------------------------------
Name......... stepper.h
Author....... Marc Adams
Date......... 24/07/03
Description.. Header file for the stepper driver
............. Ported from the Pluto 5 driver
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

#ifndef PERIPHERAL_STEPPER
#define PERIPHERAL_STEPPER
/****	Defines */

#define	RAMP_INI		0
#define	RAMP_SYS		1
#define RAMP_69			2
#define RAMP_78			3
#define RAMP_89			4
#define RAMP_96			5
#define RAMP_104		6
#define	RAMP_NUDGE1		7
#define	RAMP_NUDGE2		8
#define	RAMP_NUDGE3		9
#define RAMP_WOBBLE		10
#define RAMP_HSWOBBLE	11
#define RAMP_SKILLSLOW	12
#define RAMP_SKILLMED	13
#define RAMP_SKILLFAST	14
#define RAMP_USER		15
#define MAX_RAMPS		16

#define	TAB1_MASK	   0xFF00
#define	CORRECT_MASK   0x00FF
#define	full_step_mask 0x03		/* reels .h */
#define	half_step_mask 0x03		/* reels .h */
#define	EOF_STEPPERS   0x00

#define	SKS_STOP		0x01
#define	SKS_CORRECTED	0x02


#define	ERR_BADOPTO		0x01
#define	ERR_BADREEL		0x02

/****	Type Definitions */

typedef	struct ramp
{
	const WORD	*pattern_tbl;
	WORD		pixmask;
	void		(*const *vector_tbl)(const void*);
	const WORD	*delay_lst;
} RAMP;

typedef	struct reel_addr
{
	BYTE		byte_num;
	BYTE		high_nibble;
	INPUT		opto;
} REEL_ADDR;

typedef	struct reel_type
{
	WORD		steps_per_rev;
	WORD		steps_per_symbol;	/* steps for each symbol */
	BYTE		min_opto;			/* minimum opto count */
	BYTE		tolerance;
	BYTE		revpos;
	const RAMP	*ramps[MAX_RAMPS];	/* ramps */
} REEL_TYPE;


typedef	struct steppervars
{
	WORD		posn;
	WORD		step_dist;
	WORD		direction;
	const WORD	*pattern_tbl;
	WORD		pattern_idx;
	WORD		pixmask;

	void		(*const *vector_tbl)(const void*);
	const WORD	*delay_tbl;
	WORD		delay_idx;
	WORD		timer;

	BYTE			opto_count;
	volatile BYTE	spinning;
	BYTE			enable;

	BYTE			located;
	BYTE			pos_corrected;
	BYTE			tab_missed;
	BYTE			half_step;
} STEPPERVARS;

typedef	struct stepperzvars
{
	WORD		step_posn;
	BYTE		stop;
} STEPPERZVARS;


typedef	struct stepper
{
	const	 REEL_ADDR*		addr;		/*structure showing how to read & write to reel*/
	const 	 REEL_TYPE*		type;		/*info about the reel no. of steps etc*/
			 STEPPERVARS*	vars;		/*ram associated with the reel*/
			 STEPPERZVARS*	zvars;
	UWORD	tab_posn;					/*position information*/
} STEPPER;


typedef	struct steppercfg
{
	const	STEPPER*const*	stepperlist;
	WORD	initsteps;
	TIME	initdelay;
	TIME	optodelay;
	WORD	retries;
	TIME	retrydelay;
} STEPCFG;


typedef	struct stepvars
{
	const	STEPCFG*	config;
	BYTE				enable;
	BYTE				nofmotors;
} STEPVARS;


typedef	struct stepservices
{
	WORD		(*getnofmotors)(const	DEVICE* dev);
	void		(*initialisestepper)(const DEVICE* dev);
	void		(*restorepositions)(const DEVICE* dev);
	BYTE		(*motorsidle)(const DEVICE* dev);
	LONG		(*detectoptos)(const DEVICE* dev);

	void		(*spintotab)(const DEVICE* dev,WORD rampidx);
	void		(*spinmotor)(const DEVICE* dev,WORD motidx,WORD steps,WORD rampidx);
	void		(*stopmotor)(const DEVICE* dev,WORD motidx);
	WORD		(*getmotorposn)(const DEVICE* dev,WORD motidx);

	BYTE		(*motoridle)(const DEVICE* dev,WORD motidx);
	BYTE		(*motorspinning)(const DEVICE* dev,WORD motidx);
	WORD		(*gettabsmissed)(const DEVICE* dev,WORD motidx);
	WORD		(*getposncorrected)(const DEVICE* dev,WORD motidx);
	void		(*clearerrors)(const DEVICE* dev,WORD motidx);
	void		(*spintozero)(const DEVICE* dev);
} STEPSERVICES;

/****	Prototypes */

	#include	"stepper.p"

#endif

/*************************************************************************/
