/*-------------------------------------------------------------------
Name......... touchscreen.h
Author....... Marc Adams
Date......... 24/07/03
Description.. Header file for the generic touch screen driver
............. Ported from the Pluto 5 driver
-------------------------------------------------------------------*/

/*-------------------------------------------------------------------
DECLARATION FORMATS
Type defs........... UPPER CASE
#defines............ UPPER CASE
Global Functions.... GlobalFunction();
Static Functions.... staticfunction();
Global Variables.... GlobalVariable;
Static Variables.... staticvariable;
Local Variables..... localvariable;
Structure Elements.. Structure.element;
-------------------------------------------------------------------*/

/****	Defines */
#ifndef PERIPHERAL_TOUCHSCREEN
#define PERIPHERAL_TOUCHSCREEN

/* touchscreen comms modes */
#define COMMAND_MODE	0
#define	TOUCH_DATA_MODE	1

/* touch reporting modes */
#define DOWN_REPORT		0x01
#define STREAM_REPORT	0x02
#define UP_REPORT		0x04

/* touch event status */
#define DOWN_EVENT		0x01
#define STREAM_EVENT	0x02
#define UP_EVENT		0x04

/* reset types */
#define HARD_RESET	0
#define SOFT_RESET	1

/****	Type Definitions */
typedef struct touchidentity
{
	BYTE model[32];
	BYTE revision[32];
	BYTE features[32];
	BYTE status[32];
} TOUCHIDENTITY;

typedef	struct touchservices
{
	UWORD (*touch_screen_init)();
	void (*touch_screen_comms_mode)(BYTE mode);
	UWORD (*touch_screen_reset)(UBYTE reset_type);
	UWORD (*touch_screen_identify)(TOUCHIDENTITY *touchidentity);
	UWORD (*touch_screen_set_serial)(BYTE baud, BYTE bits, BYTE stop,
		   BYTE parity, BYTE handshake);
	UWORD (*touch_screen_set_mode)(UBYTE mode);
	UWORD (*touch_screen_calibrate)(UWORD step_number);
} TOUCHSERVICES;

typedef struct touchdata
{
	UWORD x;
	UWORD y;
	UWORD z;
	UBYTE status;
} TOUCHDATA;

typedef struct touchcalibdata
{
	UWORD x[2];
	UWORD y[2];
	UBYTE flag;
} TOUCHCALIBDATA;


/****	Prototypes */

	#include	"touchscreen.p"

#endif

/*********************************************************************/
